import random
import numpy as np
import pandas as pd


# all sensors in experiment
SENSORS_FEATURES = ['sensor_1','sensor_2','sensor_3','sensor_4','sensor_5',
 'sensor_6', 'sensor_7', 'sensor_8', 'sensor_9', 'sensor_10', 'sensor_11', 'sensor_12', 'sensor_13',
 'sensor_14', 'sensor_15', 'sensor_16', 'sensor_17', 'sensor_18', 'sensor_19', 'sensor_20', 'sensor_21', 'sensor_22', 'sensor_23', 'sensor_24']

# all stages in experiment
STAGES = ['baseline', 'absorb', 'pause', 'desorb', 'flush']

def aslist(x):
    """Checks and ensures variable is a list

    Args:
        x (str or list): 

    Returns:
        list: 
    """

    if isinstance(x, list):
        return x
    else:
        return [x]





def get_stages(stages='pause', N_STAGES=None):
    """get the stages of experiment to be used for model training

    Args:
        stages (str or list, optional): Can take any of ('baseline', 'absorb', 'pause', 'desorb', 'flush') as a str or combination as a list. 
                                        Can also take str 'all' for all stages and 'random' for random stage or stages. Defaults to 'pause'.
        N_STAGES (int, optional): if 'random' is chosen for stages, the number of stages to use. Defaults to None.

    Returns:
        list: a list of stages to be used for training
    """

    if stages== 'pause':
        return aslist('pause')
    elif stages == 'all':
        return aslist(STAGES)
    elif stages=='random':
        return np.random.choice(STAGES, N_STAGES, False).tolist()
    else:
        return stages




def get_sensors(sensors='12', N_SENSORS=None):
    """_summary_

    Args:
        sensors (str, optional): Can take any of ('sensor_1', 'sensor_2', 'sensor_3'.....'sensor_24') as a str or combination as a list. 
                                        Can also take str '24' for all sensors, '12' for 1st 12 sensors, '12r' for last 12 sensors and 'random' for random sensor or sensors. Defaults to '12'.
        N_SENSORS (_type_, optional): if 'random' is chosen for sensors, the number of sensors to use. Defaults to None.

    Returns:
        list: a list of sensors to be used for training
    """

    if sensors=='24':
        return SENSORS_FEATURES
    elif sensors=='12':
        return SENSORS_FEATURES[:12]
    elif sensors=='12r':
        return SENSORS_FEATURES[-12:]
    elif sensors=='random':
        return np.random.choice(SENSORS_FEATURES, N_SENSORS, False).tolist()
    else:
        return sensors



def get_train_features(df, sensor='12', stage='pause', use_average = False):
    
    """Selects the features to be used for model training

    Args:
        df (pandas dataframe): dataframe of cyclic sensor data
        sensor (str, optional): Can take any of ('sensor_1', 'sensor_2', 'sensor_3'.....'sensor_24') as a str or combination as a list. 
                                 Can also take str '24' for all sensors, '12' for 1st 12 sensors, '12r' for last 12 sensors and 'random' for random sensor or sensors. Defaults to '12'.        stage (str, optional): _description_. Defaults to 'pause'.
        stage (str or list, optional): Can take any of ('baseline', 'absorb', 'pause', 'desorb', 'flush') as a str or combination as a list. 
                                        Can also take str 'all' for all stages and 'random' for random stage or stages. Defaults to 'pause'.
        use_average (bool, optional): if True, uses average of stages selected. Defaults to False.


    Returns:
        data(pandas dataframe): _description_
    """

    target_col='result'

    N_SENSORS = random.randint(1,len(SENSORS_FEATURES))
    N_STAGES = random.randint(1,len(STAGES))

    stages_to_use = get_stages(stage, N_STAGES)
    sensors_to_use = get_sensors(sensor, N_SENSORS)


    if use_average:
        data = df.groupby(['measurement_stage', 'result', 'exp_unique_id']).mean().loc[stages_to_use].reset_index()[sensors_to_use+[target_col]]
        return data, sensors_to_use, stages_to_use

    else:
        data = df.loc[df['measurement_stage'].isin(stages_to_use)][sensors_to_use+[target_col]]
        return data, sensors_to_use, stages_to_use



          



















def plot_new_sample(df, x):
    
    X = df.iloc[:,1:13]
    y = df['label'].replace({'Control': 0, 'Covid': 1})#.value_counts()
    x = x.values.reshape(1,-1)


    scaler = StandardScaler()  
    scaler.fit(X.values)
    X = scaler.transform(X.values)
   
    means = np.array([243.94969928, 256.31424094, 313.33404167, 311.02738043,
        284.31009239, 307.20553442, 293.71868478, 298.39285688,
        276.90539674, 303.74205978, 262.39199638, 228.63297464])
    vars = np.array([32.38231873, 15.39840351,  0.30922943,  0.11581567, 14.18706441,
         0.67305964,  2.09758948,  0.96071839,  7.81709395,  1.53088058,
         4.84237249, 13.75799395]) 

    # means = scaler.mean_ 
    # vars = scaler.var_ 

    print('mean: ', means)
    print('var: ', vars)

    # X = scale_data(X.values, means=means,stds=vars **0.5)
    x = scale_data(x, means=means,stds=vars **0.5)

    print('scaled_new_data: ', x)

    pca = PCA(n_components = 2)
    X = pca.fit_transform(X)

    # # print('Scaled X: ', X)

    print('before pca: ', x)
    x = pca.transform(x)    
    print('after pca: ',x)

    # x = scaler.transform(x)
    # print(x)

    # pipe = Pipeline([
    #     ('scale', StandardScaler()),
    #     ('reduce_dims', PCA(n_components=2))])
    
    # pipe.fit(X.values)
    # X = pipe.transform(X.values)

    
    # print('pca_comp: ', x)




    
    group = y
    # group = y[:-10]
    cdict = {0: 'blue', 1: 'red'}
    class_label = {0: 'Control', 1: 'Covid'}

    scatter_x = X[:,0]
    scatter_y = X[:,1]

    fig, ax = plt.subplots()
    for g in np.unique(group):
        ix = np.where(group == g)
        ax.scatter(scatter_x[ix], scatter_y[ix], c = cdict[g], label = class_label[g], alpha=0.5)
    ax.scatter(x[:,0], x[:,1], c='black', label='new sample', s=70)
    ax.legend()
    ax.set_title('New data cluster assignment')
    ax.set_xlabel('PC1')
    ax.set_ylabel('PC2')
    # plt.show()
    fig.savefig('./static/img/sample_cluster.jpeg')




from scipy.signal import find_peaks
from scipy import stats
import pandas as pd
import numpy as np







FEATURES = ['sensor_1', 'sensor_2',
       'sensor_3', 'sensor_4', 'sensor_5', 'sensor_6', 'sensor_7', 'sensor_8',
       'sensor_9', 'sensor_10', 'sensor_11', 'sensor_12', 'sensor_13',
       'sensor_14', 'sensor_15', 'sensor_16', 'sensor_17', 'sensor_18',
       'sensor_19', 'sensor_20', 'sensor_21', 'sensor_22', 'sensor_23',
       'sensor_24']


def iqr(x):
    q3, q1 = np.percentile(x, [75 ,25])
    return q3 - q1

def avg_absolute_diff(x):
    return np.mean(np.absolute(x - np.mean(x)))

def max_min_diff(x):
    return np.max(x) - np.min(x)

def median_abs_dev(x):
     return np.median(np.absolute(x - np.median(x)))

def negative_count(x):
    return np.sum(x < 0)

def positive_count(x):
    return np.sum(x > 0)

def values_above_mean(x):
    return np.sum(x > x.mean())

def baseline(x):
    return x.iloc[:8].mean()

def avg_of_pause(x):
    return x.iloc[[37,38,39,40]].mean()

def pause_phase_area(x):
    x = x.iloc[[37,38,39,40]]
    return np.trapz(x) 


def peaks(x):
    return len(find_peaks(x)[0])


def extract_features(df, unique_id='exp_unique_id', label='result', features=FEATURES):

    features = features + [unique_id, label]
    df = df[features].groupby(by=[unique_id, label]).agg([min, max, np.std, iqr, np.median, stats.skew, stats.kurtosis, \
    np.trapz, avg_absolute_diff, peaks, values_above_mean, positive_count, negative_count, median_abs_dev, \
    max_min_diff, baseline, avg_of_pause, pause_phase_area]).reset_index()[features]

    df.columns = ["_".join(col) for col in df.columns.to_flat_index()]
    df = df.rename(columns={'result_':'result'}).drop('exp_unique_id_', axis=1)
    return df










def load_ml_model(path_to_model):
  model = pickle.load(open(path_to_model, "rb"))
  return model

def random_sample(df_list,x_columns):
  i = random.randint(0, len(df_list))
  df_temp = df_list[i]
  df_temp = df_temp[x_columns]
  # df_temp = df_temp.iloc[:,[1,2,3,4,5,6,7,8,9,10,11,12]]
  j = random.randint(0, len(df_temp.count()))
  input_data_sample = df_temp.iloc[[j]]
  print(input_data_sample)
  # 
  x = input_data_sample.columns.tolist()
  y = input_data_sample.iloc[0].values.tolist()
  # 
  plt.bar(x, y)
  fig = plt.gcf()
  fig.autofmt_xdate()
  # 
  return input_data_sample

def sample_prediction(input_data_sample, model):
  # 
  print('Prediction: ' + str(model.predict(input_data_sample)))
  print('Prediction Probability: ' + str(model.predict_proba(input_data_sample)))